import string

from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.PythonScripts.standard import html_quote
from Globals import MessageDialog, Persistent
from OFS import SimpleItem, ObjectManager


from NetSendUtils import *

    

manage_addNetSendForm = PageTemplateFile('netsendform', globals())

def manage_addNetSend(self,id="netsend",
                      title="NetSend", avoidusers=[],
                      skin='default',homeaddr=['127.0.0.1'],
                      REQUEST=None):
    """ add netsend object """
    self._setObject(id, Netsend(id, title, avoidusers, skin, homeaddr))
    if REQUEST is not None:
        return MessageDialog(title='NetSend object Added',
              message='This object now has a NetSend object', 
              action=REQUEST['URL1']+'/manage_main')

class Netsend(ObjectManager.ObjectManager,
              SimpleItem.SimpleItem,
              Persistent):
    """ Netsend class """
    
    meta_type='NetSend'


    
    manage_netsend = PageTemplateFile('manage_netsend', globals())
    
       
    manage_main = manage_workspace = manage = manage_netsend
    # for debugging purposes, let Contents be the first View
    
    manage_options = (
        {'label': 'Properties','action':'manage_netsend'},
        {'label': 'Contents', 'action': 'manage_main'},
        {'label': 'View', 'action': 'index_html'}
        )
    def __init__(self,id,title,avoidusers,skin,homeaddr,alias={}):
        self.id = id
        self.title = title
        self.avoidusers=nonEmptyList(avoidusers)
        self.skin = skin
        if type(homeaddr)==type('s'):homeaddr=[homeaddr]
        self.homeaddr = homeaddr
        self.alias = alias

    def manage_edit(self,title, avoidusers=None,skin=None,homeaddr=None,REQUEST=None):
        """ edit the object """
        self.title = title
        if avoidusers is not None:
            self.avoidusers = avoidusers
        if skin is not None:
            self.skin = skin
        if homeaddr is not None:
            if type(homeaddr)==type('s'):homeaddr=[homeaddr]
            self.homeaddr = homeaddr

        
        
        return MessageDialog(title='NetSend object changed',
              message='Properties saved', 
              action=REQUEST['URL1']+'/manage_main')

    def index_html(self):
        " "

        if hasattr(self,'skin') and self.skin != 'default':
            skinfile=self.skin
        else:
            # default
            skinfile=""
        
        skinfile = string.replace(skinfile,'.','/')+'/index'
        if skinfile[0]=='/':skinfile = skinfile[1:]

        return PageTemplateFile(skinfile, globals()).__of__(self)()

    def listSkins(self):
        """ list all skins in the Products folder """
        skins={}
        skinpath = os.path.join(os.path.dirname(__file__),'skins')

        for file in os.listdir(skinpath):
            # is it a folder?
            if os.path.isdir(os.path.join(skinpath,file)):
                id,title = filename2id_title(file)
                skins["skins.%s"%file] = title

        first=''
        available=[]
        
        for skinid,skintitle in skins.items():
            if hasattr(self,'skin') and self.skin == skinid:
                first=[skinid, skintitle, 1]
            else:
                available.append([skinid,skintitle,0])
        if first=='':
            first=['','Default',1]
        else:
            available.insert(0,['','Default',0])
                
        return ([first]+available)

    def sniffSendmessage(self, REQUEST):
        """ sends message only if request has proper variables
        consider this a wrapper to sendmessage() for templates """
        
        if REQUEST.has_key('usernames') and REQUEST.has_key('message') \
           and type(REQUEST['usernames'])==type([]) and len(REQUEST['usernames'])>0\
           and REQUEST['message'] !='':
            try:
                return self.sendmessage(REQUEST['usernames'],REQUEST['message'])
            except:
                return invokeJavaScriptError()+"""<strong><font color="red">Bad!</font> You can't do that!</strong><br> Get your own life, then try later."""

        return ""

    def IPAllowed(self):
        " return if REQUEST IP in self.homeaddr list "
        return self.REQUEST.REMOTE_ADDR in self.homeaddr
    
    def sendmessage(self,usernames,message):
        """ sends the message on the commandline """
        if not self.IPAllowed():
            raise "Not You", "Only the host can send messages from here"
        
        if message=='':
            raise "EmptyMessage", "No content in message"

        if type(usernames)==type(''):
            usernames=[usernames]

        realnames = self.getUsernames(realnames=1)
        
        c=0
        success_users=[]
        for username in usernames:
            if username in realnames:
                try:
                    
                    result = sendMessageChecked(username,message)
                    c=c+1
                    success_users.append(username)
                    self.savemessage(username,message)
                except:
                    self.savemessage(username,message,success=0)
            else:
                self.savemessage(username,message,success=0)
            

                

        return "%s messages sent to %s"%(c,string.join(self.getAliasname(usernames),', '))


    def getUsernames(self,realnames=0,everybody=0,asdict=0,realnameinbrackets=0):
        """ return
            list of lists [['pbn','Peter']]
            or just usernames
            or as dictionary {'pbn':'Peter'}
        """
        skips=self.avoidusers

        ntusers = fetchNetUsers()
        users={}
        for ntuser in ntusers:
            if ntuser not in skips or everybody:
                if self.alias.has_key(ntuser):
                    users[ntuser] = self.alias[ntuser]
                else:
                    # alias doesn't exist for this user
                    users[ntuser] = ntuser

        if realnames:
            # want only the realnames. Suit yourself
            return users.keys()
        elif asdict:
            # you want it as a dictionary
            return users
        elif realnameinbrackets:
            # you want them as a special list
            # and displayname with the realname in bracket
            nlist=[]
            for realname,displayname in users.items():
                
                if realname!=displayname:
                    displayname = "%s (%s)"%(displayname,realname)
                    
                nlist.append([realname,displayname])
            return nlist
        else:
            # you want them as a special list
            nlist=[]
            for realname,displayname in users.items():
                nlist.append([realname,displayname])
            return nlist
            

    def getAliasname(self,realnames):
        """ a wrapper to getAlias_fromAlias()
        this one returns the realname back if getAlias_fromAlias() fails
        """

        if type(realnames)==type('s'):
            try:
                return self.getAlias_fromAlias(realnames)
            except:
                # return back
                return realnames
        else:
            aliases=[]
            for realname in realnames:
                try:
                    aliases.append(self.getAlias_fromAlias(realname))
                except:
                    # same
                    aliases.append(realname)
            return aliases
                
            
    def getName_fromAlias(self,alias):
        """ the property self.alias is a dicationary like: {'pbn':'Peter'}
        this method returns 'pbn' from 'Peter'
        """
        for k,v in self.alias.items():
            if v==alias:
                return k
            
        # Uh? still here?
        raise "NonExistantUser", "%s doesn't exist in aliases"%alias

    def getAlias_fromAlias(self,name):
        """ the property self.alias is a dicationary like: {'pbn':'Peter'}
        this method returns 'Peter' from 'pbn'
        """
        try:
            return self.alias[name]
        except KeyError:
            raise "NonExistantUser", "%s doesn't exist in aliases"%name
    
    
    def manage_editalias(self,realname,displayname,REQUEST=None):
        """ change alias of a user """

        if realname not in self.getUsernames(realnames=1):
            raise "NonExistantUser", "%s doesn't exist as a user"%realname
        else:
            if displayname!='':
                self.alias[realname] = displayname
           

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(REQUEST.URL1+'/manage_main')
        return "Alias changed"

    
    def savemessage(self,username,message,success=0):
        """ classwrapper
        I'm not quite sure why this is needed but it helps """
        manage_addNetSendMessage(self,username,message,success)
        return "message saved"


    def getMessages(self):
        """ return the log messages """
        objects= self.objectValues('NetSendMessage')
        objects.reverse()
        return objects

    def cooklogmessage(self,message,username,bobobasetime,msglen=60):
        """ return the display in the form of a message """
        usersstring = self.getAliasname(username)

        if len(usersstring) >= 20:
            usersstring = "%s..."%usersstring[:20]

        if len(message) >= msglen:
            message = "%s..."%string.strip(message[:msglen])
        
        out = usersstring + ' - ' + bobobasetime().strftime('%Y/%m/%d')

        out = out + ' - %s' % message
        
        return html_quote(out)

    def sniffLogmessage(self,REQUEST):
        """ do stuff with the log messages depending on what's
        in the REQUEST. """
        if REQUEST.has_key('messages') and type(REQUEST['messages'])==type([])\
           and len(REQUEST['messages'])>0:
            if REQUEST.has_key('deletemessages'):

                count = len(REQUEST['messages'])
                self.manage_delObjects(REQUEST['messages'])

                REQUEST.RESPONSE.redirect(REQUEST.URL1+'?messagesremoved=%s'%count)
            else:
                logobjects=[]
                # base looping on self.objectValues() instead of looping
                # on REQUEST['messages'] to get the proper order
                for object in self.objectValues('NetSendMessage'):
                    if object.id in REQUEST['messages']:
                        logobjects.append(object)
                logobjects.reverse()
                return logobjects

        elif REQUEST.has_key('clearallmessages'):
            objectIds = self.objectIds('NetSendMessage')
            count = len(objectIds)
            self.manage_delObjects(objectIds)
            REQUEST.RESPONSE.redirect(REQUEST.URL1+'?messagesremoved=%s'%count)

        # still here? (default)
        return []

        

def manage_addNetSendMessage(self,username,message,success=0):
    """ add NetSendMessage object """
    id = generateRandomMessageId()
    message = string.replace(message,'\\"','"')
    self._setObject(id, NetSendMessage(id,username,message,success))
    
    
    
class NetSendMessage(SimpleItem.SimpleItem):
    """ NetSend message, the log of NetSend """
    meta_type='NetSendMessage'
    

    def __init__(self,id,username,message,success=1):
        " "
        self.id = id
        if type(username)==type([]):
            username= username[0]
        self.username = username
        self.message = message
        self.success = success

    def __str__(self):
        """ info about the log """
        if self.success:
            success_failure="message sent successfully"
        else:
            success_failure="failed to send message"
        return """\
        NetSendMessage <id: %s>
        %s

        username: %s

        %s"""%(self.id, success_failure, self.username, self.message)

    index_html = manage = manage_workspace = manage_main = __str__
        
        

